#include "stdafx.h"

#include <windows.h>
#include <GL/gl.h>
#include <vcg/CIMsg.h>

void CIMsg::showError(const char* m) {showError(m, "Error");}
void CIMsg::showWarning(const char* m) {showWarning(m, "Warning");}
void CIMsg::showInfo(const char* m) {showInfo(m, "Info");}
void CIMsg::showLastError(const char* m) {showLastError(m, "Error");}
void CIMsg::showLastWarning(const char* m) {showLastWarning(m, "Warning");}
void CIMsg::checkGLError(const char* m) {checkGLError(m, "OpenGL Usage Error");}


void CIMsg::showError(const char* message, const char* title) {

  MessageBox( NULL, message, title, MB_OK | MB_ICONEXCLAMATION );
}


void CIMsg::showWarning(const char* message, const char* title) {

  MessageBox( NULL, message, title, 
	      MB_OK | 
	      MB_ICONEXCLAMATION);
}

void CIMsg::showInfo(const char* message, const char* title) {

  MessageBox( NULL, message, title, MB_OK | MB_ICONINFORMATION );
}



void CIMsg::showLastError(const char* m, const char* title) {

	LPVOID lpMsgBuf;
	FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | 
				   FORMAT_MESSAGE_FROM_SYSTEM |
				   FORMAT_MESSAGE_IGNORE_INSERTS,
				   NULL,
				   GetLastError(),
				   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				   (LPTSTR) &lpMsgBuf,
				   0,
				   NULL );

	CString message(m);
	message+=((LPCTSTR)lpMsgBuf);
	LocalFree( lpMsgBuf );
	showError(message, title);
}

void CIMsg::showLastWarning(const char* m, const char* title) {

	LPVOID lpMsgBuf;
	FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | 
				   FORMAT_MESSAGE_FROM_SYSTEM |
				   FORMAT_MESSAGE_IGNORE_INSERTS,
				   NULL,
				   GetLastError(),
				   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				   (LPTSTR) &lpMsgBuf,
				   0,
				   NULL );

	CString message(m);
	message+=((LPCTSTR)lpMsgBuf);
	LocalFree( lpMsgBuf );
	showWarning(message, title);
}


void CIMsg::checkGLError(const char* m, const char* title) {

  CString message(m);

  switch(glGetError()) {

  case GL_NO_ERROR:
    return;

  case GL_INVALID_ENUM:
    message+=(": invalid enum");
    showWarning(message, title);
    break;

  case GL_INVALID_VALUE:
    message+=(": invalid value");
    showWarning(message, title);
    break;

  case GL_INVALID_OPERATION:
    message+=(": invalid operation");
    showWarning(message, title);
    break;

  case GL_STACK_OVERFLOW:
    message+=(": stack overflow");
    showWarning(message, title);
    break;

  case GL_STACK_UNDERFLOW:
    message+=(": stack underflow");
    showWarning(message, title);
    break;

  case GL_OUT_OF_MEMORY:
    message+=(": out of memory");
    showWarning(message, title);
    break;
  }

}
